@@ -56,12 +56,14 @@ MainWindow::MainWindow(QWidget *parent) :
 void MainWindow::setupActions() {
     KStandardGameAction::gameNew(this, SLOT(startRandomGame()), actionCollection());
     KStandardGameAction::load(this, SLOT(loadBoard()), actionCollection());
+    KStandardGameAction::restart(this, SLOT(restartGame()), actionCollection());
     KStandardGameAction::highscores(this, SLOT(highscores()), actionCollection());
     KStandardGameAction::quit(this, SLOT(close()), actionCollection());
     KStandardAction::preferences(this, SLOT(settings()), actionCollection());
     KStandardAction::fullScreen(this, SLOT(toggleFullscreen(bool)), this, actionCollection());
     m_action_pause = KStandardGameAction::pause(this, SLOT(togglePaused(bool)), actionCollection());
     m_action_undo = KStandardGameAction::undo(this, SLOT(undo()), actionCollection());
+    m_action_hint = KStandardGameAction::hint(this, SLOT(hint()), actionCollection());
 
     m_action_save_state = actionCollection()->addAction("save-position");
     m_action_save_state->setText(i18n("Save Position"));
@@ -157,6 +159,13 @@ void MainWindow::undo() {
     m_scene->refresh(p);
 }
 
+void MainWindow::hint()
+{
+    QPoint p = m_game->hint();
+    m_scene->refresh(p);
+    m_scene->hover(p.x(), p.y());
+}
+
 void MainWindow::saveState() {
     m_game->saveState();
     updatePositions();
@@ -174,6 +183,13 @@ void MainWindow::startRandomGame() {
     startGame();
 }
 
+void MainWindow::restartGame()
+{
+    m_game = QSharedPointer<Picmi>(new Picmi(m_game->getBoardMap()));
+
+    startGame();
+}
+
 void MainWindow::startPresetGame(QSharedPointer<Level> board) {
     QSharedPointer<BoardMap> p(new BoardMap(board->width(), board->height(), board->map()));
     m_game = QSharedPointer<Picmi>(new Picmi(p));
@@ -190,6 +206,7 @@ void MainWindow::startGame() {
     }
 
     m_action_undo->setEnabled(false);
+    m_action_hint->setEnabled(true);
     m_action_save_state->setEnabled(true);
     m_action_load_state->setEnabled(false);
     m_action_pause->setEnabled(true);
@@ -237,6 +254,7 @@ void MainWindow::gameWon() {
     KScoreDialog::FieldInfo score = m_game->endGame();
     m_view.setEnabled(false);
     m_action_pause->setEnabled(false);
+    m_action_hint->setEnabled(false);
     m_action_undo->setEnabled(false);
     m_action_save_state->setEnabled(false);
     m_action_load_state->setEnabled(false);
@@ -276,6 +294,7 @@ void MainWindow::highscores() {
 void MainWindow::togglePaused(bool paused) {
     m_view.setPaused(paused);
     m_action_undo->setEnabled(!paused);
+    m_action_hint->setEnabled(!paused);
     m_action_save_state->setEnabled(!paused);
     m_action_load_state->setEnabled(!paused);
 
