@@ -57,6 +57,7 @@ public:
     int width() const;
     int height() const;
     int remainingBoxCount() const { return m_map->boxCount() - m_state->boxCount(); }
+    QSharedPointer<BoardMap> getBoardMap() const { return m_map; }
     bool outOfBounds(int x, int y) const;
 
     /* 0 <= x < width(); 0 <= y < height() */
@@ -72,6 +73,8 @@ public:
     /* undo last action (if it exists) and return the changed coordinate. */
     QPoint undo();
 
+    QPoint hint();
+
     /* if a saved state exists, load it. otherwise, do nothing */
     void loadState() { m_state->loadState(); m_streaks->update(); emit stateChanged(); }
     void saveState() { m_state->saveState(); }
